// Copyright 2019 Diffuse Logic, all rights reserved.

#ifndef DL_CORE_LOGGING_H
#define DL_CORE_LOGGING_H

#include "dl_string.h"

namespace dl {

enum LogType
{
    LogType_Info,
    LogType_Warning,
    LogType_Error,
    LogType_Custom
};

// Callback for logging functions.
//
typedef void( *pLogFunc )( void*, LogType, const char* );

// Subscribe to be called back for logging.
//
void DL_API     subscribeLog( void* ctx, pLogFunc func );
void DL_API     unsubscribeLog( void* ctx );

void DL_API     openLogFile( String path = "" );
void DL_API     closeLogFile();

// Call this when your console is ready to print messages.
//
void DL_API     flushStartupMessages();

// Call these at any time.
//
void DL_API     logMessage( LogType type, String msg );
void DL_API     logMessage( LogType type, const char* fmt, ... );
void DL_API     logMessagev( LogType type, const char* fmt, va_list ap );
void DL_API     logMessageTsv( LogType type, const char* fmt, va_list ap ); // Add timestamp

void DL_API     logInfo( const char* fmt, ... );
void DL_API     logWarning( const char* fmt, ... );
void DL_API     logError( const char* fmt, ... );
void DL_API     logCustom( const char* fmt, ... );

void DL_API     logInfoTs( const char* fmt, ... );
void DL_API     logWarningTs( const char* fmt, ... );
void DL_API     logErrorTs( const char* fmt, ... );
void DL_API     logCustomTs( const char* fmt, ... );

void DL_API     logNewLine();
void DL_API     logSeparator( );
void DL_API     logBanner( const char* fmt, ... );

// Console color.
//
enum ConsoleColor
{
    ConsoleColor_Default,
    ConsoleColor_Red,
    ConsoleColor_Yellow,
    ConsoleColor_Cyan
};

void DL_API     setConsoleColor( ConsoleColor color );
void DL_API     resetConsoleColor();

#define DL_PRINT_COLOR( CONSOLE_COLOR, FMT, ... ) \
    dl::setConsoleColor( CONSOLE_COLOR ); \
    fprintf( stdout, FMT, __VA_ARGS__ ); \
    dl::resetConsoleColor();

#define DL_PRINT( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Default, FMT, __VA_ARGS__ )

#define DL_PRINT_RED( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Red, FMT, __VA_ARGS__ )

#define DL_PRINT_YELLOW( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Yellow, FMT, __VA_ARGS__ )

#define DL_PRINT_CYAN( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Cyan, FMT, __VA_ARGS__ )

} // dl

#endif // DL_CORE_LOGGING_H
